/* Copyright 2010 Santiago Ontanon and Ashwin Ram */

package d2.execution.planbase;

import d2.core.D2;
import gatech.mmpm.GameState;

import gatech.mmpm.sensor.Sensor;
import gatech.mmpm.util.XMLWriter;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.jdom.Element;

import d2.plans.ActionPlan;
import d2.plans.Plan;
import d2.plans.PlanState;


public abstract class PlanBase {
        D2 m_d2;
	protected List<PlanBaseEntry> m_plans;
	List<ActionPlan> m_allActions = null;
	int DEBUG = 2;
	
	public PlanBase(D2 d2) {
            m_d2 = d2;
            m_plans = new LinkedList<PlanBaseEntry>();
	}

        public void setD2(D2 d2) {
            m_d2 = d2;
        }

        public D2 getD2() {
            return m_d2;
        }
	
	public void addPlan(PlanBaseEntry p) {
		m_plans.add(p);
	}

	public void addPlans(List<PlanBaseEntry> l) {
		m_plans.addAll(l);
	}
	
	public PlanBaseEntry getPlan(int index) {
		return m_plans.get(index);
	}
	
	public String toString() {
		return "PlanBase, size = " + m_plans.size() + ".";
	}
	
	public abstract Plan retrievePlan(Sensor g, int cycle, GameState gs, String player);
	public abstract List<Plan> retrieveNPlans(Sensor g, int cycle, GameState gs, String player, int N);
	
	// This method goes over all the actions in the plans in the base, and tries to find one that
	// individually satisfied a particular goal.
	public abstract ActionPlan retrieveAction(Sensor g,Plan target,int cycle, GameState gs, String player);
	
	public List<Plan> getPlans() {
		List<Plan> l = new LinkedList<Plan>();
		for(PlanBaseEntry pbe:m_plans) {
			l.add(pbe.m_plan);
		}
		return l;
	}

	public abstract void saveToXML(XMLWriter w) throws IOException;

	public static PlanBase loadFromXML(Element xml,String domain, D2 d2) {
		String classString = xml.getChildText("type");

		// TODO: this is a hack, find a better way:
		if (classString.equals("SimilarityRetrieval")) return SimilarityRetrieval.loadFromXMLInternal(xml,domain, d2);
		
		return null;
	}
	
	public List<ActionPlan> getAllActions() {
		if (m_allActions == null) {
			m_allActions = new LinkedList<ActionPlan>();
			for(PlanBaseEntry pbe:m_plans) {
				for(PlanState ps:pbe.m_plan.getAllPlanStates()) {
					if (ps.getPlan() instanceof ActionPlan) m_allActions.add((ActionPlan)(ps.getPlan()));
				}
			}			
		}
		return m_allActions;
	}

}
