/* Copyright 2010 Santiago Ontanon and Ashwin Ram */

package d2.execution.planbase;

import d2.core.D2;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import org.jdom.Element;

import d2.worldmodel.ConditionMatcher;
import d2.plans.ActionPlan;
import d2.plans.PetriNetPlan;
import d2.plans.Plan;
import d2.plans.State;
import gatech.mmpm.Context;
import gatech.mmpm.GameState;

import gatech.mmpm.sensor.Sensor;
import gatech.mmpm.sensor.composite.Invocation;
import gatech.mmpm.util.Pair;
import gatech.mmpm.util.XMLWriter;

public class SimilarityRetrieval extends PlanBase {

	public SimilarityRetrieval(D2 d2) {
            super(d2);
	}
	
	public Plan retrievePlan(Sensor g, int cycle, GameState gs, String player) {
		if (DEBUG >= 1) System.out.println("Inside SimilarityRetrieval:retrievePlan: " + m_plans.size() + " planBaseEntries");
		PetriNetPlan best = null;
		HashMap<String,Integer> best_entryPoint = null;
		double best_similarity = 0;
		GameStateFeatures gsf = new GameStateFeatures(cycle, gs,player);
		GameStateFeatures gsf_cb = null;

		double similarity;

        if (DEBUG>=2) {
            System.out.println(gsf);
        }

		for(PlanBaseEntry pbe:m_plans) {
			HashMap<String,Integer> entryPoint = pbe.m_entryPoint;
			double match = m_d2.getWorldModel().getConditionMatcher().match((Context)null,pbe.m_goal, cycle, gs, player,
                                                                            null,g, 0, pbe.m_gameState, pbe.m_originalPlayer);
			if (DEBUG >= 2) System.out.println("matching: " + g + " with " + pbe.m_goal + " -> " + match);
			if (match>0.0) {
				gsf_cb = (GameStateFeatures) pbe.m_gameState.getMetaData("GameStateFeatures");
				if (gsf_cb==null) {
					gsf_cb = new GameStateFeatures(cycle, pbe.m_gameState,pbe.m_originalPlayer);
					pbe.m_gameState.addMetaData("GameStateFeatures", gsf_cb);
				}
				similarity = GameStateSimilarity.gameStateSimilarity(gsf_cb,gsf,null);
				if (best==null || similarity>best_similarity) {
					best = pbe.m_plan;
					best_entryPoint = entryPoint;
					best_similarity = similarity;
				}
			}
		}

		if (best!=null) {
			// Clone plan:
			PetriNetPlan clone = (PetriNetPlan)best.clone();

			// Apply entry point:
			for(String s:best_entryPoint.keySet()) {
				State state = (State)clone.getPetriNetElement(s);
				state.setCurrentNumberOfTokens(best_entryPoint.get(s));
			}

			return clone;

		} else {
			return null;
		}
	}

	public List<Plan> retrieveNPlans(Sensor g, int cycle, GameState gs, String player, int N)
	{
		if (DEBUG >= 1) System.out.println("Inside SimilarityRetrieval:retrieveNPlans: " + m_plans.size() + " planBaseEntries");
		List <Pair<PlanBaseEntry,Double>> bestPlans = new ArrayList <Pair<PlanBaseEntry,Double>> ();		
		GameStateFeatures gsf = new GameStateFeatures(cycle, gs,player);
		GameStateFeatures gsf_cb = null;

		double similarity;

        if (DEBUG>=2) {
            System.out.println(gsf);
        }

        for(PlanBaseEntry pbe:m_plans) {
			double match = m_d2.getWorldModel().getConditionMatcher().match((Context)null,pbe.m_goal, cycle, gs, player,
                                                                            null,g, 0, pbe.m_gameState, pbe.m_originalPlayer);
			if (DEBUG >= 2) System.out.println("matching: " + g + " with " + pbe.m_goal + " -> " + match);
			if (match>0.0) {
				gsf_cb = (GameStateFeatures) pbe.m_gameState.getMetaData("GameStateFeatures");
				if (gsf_cb==null) {
					gsf_cb = new GameStateFeatures(cycle, pbe.m_gameState,pbe.m_originalPlayer);
					pbe.m_gameState.addMetaData("GameStateFeatures", gsf_cb);
				}
				similarity = GameStateSimilarity.gameStateSimilarity(gsf_cb,gsf,null);
				if (DEBUG>=1) System.out.println("retrieveNPlans: similarity " + similarity);

				bestPlans.add(new Pair<PlanBaseEntry,Double>(pbe,similarity));				
			}
		}
		List <Plan> returnedPlansList = new ArrayList <Plan>();
		
		// sort the plans:
		{
			boolean changes = true;
			while(changes) {
				changes = false;
				for(int i = 0;i<bestPlans.size()-1;i++) {
					Pair<PlanBaseEntry,Double> p1 = bestPlans.get(i);
					Pair<PlanBaseEntry,Double> p2 = bestPlans.get(i+1);
					
					if (p1._b<p2._b) {
						PlanBaseEntry tmp_a = p1._a;
						Double tmp_b = p1._b;
						p1._a = p2._a;
						p1._b = p2._b;
						p2._a = tmp_a;
						p2._b = tmp_b;
						changes = true;
					}
				}
			}
		}
		
		if (DEBUG>=2) {
			System.out.println("Sorted list of plans:");
			for(Pair<PlanBaseEntry,Double> bPlan : bestPlans) {
				System.out.println(bPlan._b);
			}
		}
		
		while(bestPlans.size()>N) bestPlans.remove(bestPlans.size()-1);
		
		for ( Pair<PlanBaseEntry,Double> bPlan : bestPlans)
		{
			//Clone the plan
			PetriNetPlan clone = (PetriNetPlan)(bPlan._a.m_plan.clone());
										
			// Apply entry point:
			for(String s:bPlan._a.m_entryPoint.keySet()) {
				State state = (State)clone.getPetriNetElement(s);
				state.setCurrentNumberOfTokens(bPlan._a.m_entryPoint.get(s));
			}
			
			returnedPlansList.add(clone);
		}
		return returnedPlansList;

	}
	
	
	public void saveToXML(XMLWriter w) throws IOException {
		w.tag("PlanBase");
		w.tag("type",this.getClass().getSimpleName());
		w.tag("plans");
		for(PlanBaseEntry pbe:m_plans) {
			pbe.saveToXML(w);
		}
		w.tag("/plans");
		w.tag("/PlanBase");
	}

	
	public static PlanBase loadFromXMLInternal(Element xml,String domain, D2 d2) {
		SimilarityRetrieval ret = new SimilarityRetrieval(d2);
		
		Element plans = xml.getChild("plans");
		List plan_l = plans.getChildren();
		for(Object o:plan_l) {
			Element plan_e = (Element)o;
			ret.m_plans.add(PlanBaseEntry.loadFromXML(plan_e,domain));
		}
		
		
		return ret;
	}

	
	// The action is assumed to be retrieved to satisfy a particular Sensor "g" with is a precondition of "target"
	public ActionPlan retrieveAction(Sensor raw_g,Plan target,int cycle, GameState gs, String player) {
		List<ActionPlan> actions = getAllActions();
		
//		System.out.println("retrieveAction: candidate actions: " + m_allActions.size());
        Sensor g = raw_g;
        if (g instanceof Invocation) {
            g = ((Invocation)raw_g).getSensor();
        }

		ActionPlan best = null;
		double best_score = 0;
		GameStateFeatures gsf = new GameStateFeatures(cycle, gs,player);
		GameStateFeatures gsf_cb = null;

		double match;
		double similarity;
		double score;

		for(ActionPlan a:actions) {
			List<Sensor> cl = new LinkedList<Sensor>();
			d2.execution.adaptation.ExpandConditionMatcher.expandCondition(a.getPostCondition(), cl);
			d2.execution.adaptation.ExpandConditionMatcher.expandCondition(a.getSuccessCondition(), cl);
//			System.out.println("Action: " + a);
			for(Sensor c:cl) {
//				System.out.println("Matching: " + c + " vs " + g);
				match = m_d2.getWorldModel().getConditionMatcher().match(a,c, 0, a.getOriginalGameState(), a.getOriginalPlayer(),
                                                                         target,g, cycle, gs,player);
				if (match>0.0) {
					gsf_cb = (GameStateFeatures) a.getOriginalGameState().getMetaData("GameStateFeatures");
					if (gsf_cb==null) {
						gsf_cb = new GameStateFeatures(cycle, a.getOriginalGameState(),a.getOriginalPlayer());
						a.getOriginalGameState().addMetaData("GameStateFeatures", gsf_cb);
					}
					similarity = GameStateSimilarity.gameStateSimilarity(gsf_cb,gsf,null);
					score = 0.33*similarity + 0.67*match;
//					System.out.println("Match!!!!: " + match + " " + similarity + " -> " + score);
					if (best==null || score>best_score) {
						best = a;
						best_score = score;
					}
				}			
			}
		}
		
		return best;
	}

}
