/* Copyright 2010 Santiago Ontanon and Ashwin Ram */

package d2.learn.planlearning.goalrec;

import d2.execution.planbase.GameStateSimilarity;
import org.jdom.Element;

import d2.execution.planbase.PlanBase;

import gatech.mmpm.ActionParameterType;
import gatech.mmpm.GameState;

import gatech.mmpm.sensor.Sensor;
import gatech.mmpm.util.XMLWriter;

/**
 * @author kane
 * 
 */
public class DeltaCondition extends Sensor {

	private GameState first;

	private GameState second;

	private String domain;

	public DeltaCondition(GameState g1, GameState g2, String dom) {
		first = g1;
		second = g2;
		domain = dom;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sensors.Condition#clone()
	 */
	@Override
	public Object clone() {
		// TODO Auto-generated method stub
		return new DeltaCondition(first, second, domain);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see sensors.Condition#evaluate(int, domain.GameState, java.lang.String)
	 */
	@Override
	public Object evaluate(int cycle, GameState gs, String player, gatech.mmpm.Context parameters) {
		double delta1 = GameStateSimilarity.gameStateSimilarity(first, second, 0, player, player, null);
		double delta2 = GameStateSimilarity.gameStateSimilarity(first, gs, 0, player, player, null);
		double difference = Math.abs(delta1 - delta2);
		return 1.0 - difference;
	}
	
	public ActionParameterType getType() {
		return ActionParameterType.BOOLEAN;
	}

	public void writeToXML(XMLWriter w) {
		w.tagWithAttributes("Sensor", "type = '" + this.getClass().getName()+ "'");
		first.writeToXML(w);
		second.writeToXML(w);
		w.rawXML(" <Domain>" + this.getClass().getName() + "</Domain>");
		w.rawXML(" </Sensor>");
	}

	public static Sensor loadFromXMLInternal(Element xml) {
		String domain = xml.getChildText("Domain");
		Element c1_xml = (Element) xml.getChildren("gamestate").get(0);
		GameState c1 = GameState.loadFromXML(c1_xml, d2.core.Config.getDomain());
		Element c2_xml = (Element) xml.getChildren("gamestate").get(1);
		GameState c2 = GameState.loadFromXML(c2_xml, d2.core.Config.getDomain());
		return new DeltaCondition(c1, c2, domain);
	}

}
