/* Copyright 2010 Santiago Ontanon and Ashwin Ram */

package d2.learn.planlearning.goalrec;

import java.util.List;

import gatech.mmpm.Entity;
import gatech.mmpm.GameState;

/**
 * @author kane
 * 
 * Class for determining the difference between two GameStates.
 */
public class GameStateDelta {
	/**
	 * The difference between two GameStates as a real-valued number between 0.0
	 * and 1.0, with 0.0 being absolutely the same and 1.0 being totally
	 * different.
	 */
	private double delta = 1.0;

	/**
	 * The first gamestate.
	 */
	private GameState g1 = null;

	/**
	 * The second gamestate.
	 */
	private GameState g2 = null;
	
	private static final int DEBUG = 0; 
	
	/**
	 * Default constructor.
	 * @param first one GameState.
	 * @param second the other GameState.
	 */
	public GameStateDelta(GameState first, GameState second) {
		g1 = first;
		g2 = second;
	}

	/**
	 * Calculate the delta between the two gamestates.
	 * @return
	 */
	public double findDelta(String player) {
		if (null == g1 && null == g2) {
			delta = 0;
		} else if (null == g1) {
			delta = 1;
		} else if (null == g2) {
			delta = 1;
		} else {
			// real work here
			delta = 1;
			List<Entity> entities = g1.getAllEntities();
			List<Entity> otherEntities = g2.getAllEntities();
			
			
			GameState other = g2;
			// use the bigger list as  master list of entities
			if (entities.size() < otherEntities.size()) {
				other = g1;
				List<Entity> temp = entities;
				entities = otherEntities;
				otherEntities = temp;
			}
			
			//check entities
			double entityPortion = 1.0 / entities.size();
			for (Entity e : entities) {
				Entity e2 = other.getEntity(e.getentityID());
				
				if (e2 == null) {
					//definite change here, don't lower delta
					continue;
				}
				
				//check features
				int featCount = e.listOfFeatures().size();
				for (String feature : e.listOfFeatures()) {
					if (DEBUG >= 2) {
						System.out.println("GameStateDelta.findDelta:" + e);
						System.out.println("GameStateDelta.findDelta:" + feature);
						System.out.println("GameStateDelta.findDelta:" + e2);
						System.out.println("GameStateDelta.findDelta:" + e2.featureValue(feature));
						System.out.println("GameStateDelta.findDelta:" + e.featureValue(feature));
					}
					if (e.featureValue(feature) == null && e2.featureValue(feature) == null) {
						delta -= entityPortion / featCount;
					} else if (e.featureValue(feature) == null) {
						//no change to delta
					} else if (e.featureValue(feature).equals(e2.featureValue(feature))) {
						delta -= entityPortion / featCount;
					}
				}
			}
		}
		
		if (DEBUG >= 1) {
			System.out.println("GameStateDelta.findDelta: myDelta = " + delta);
		}
		//delta = PlanBase.gameStateSimilarity(g1, g2, 0, player, null);
		if (DEBUG >= 1) {
			System.out.println("GameStateDelta.findDelta: thierDelta = " + delta);
		}
		return delta;
	}

	/**
	 * getter for the delta.
	 * 
	 * @return the difference between the GameStates.
	 */
	public double getDelta() {
		return delta;
	}

}
