/* Copyright 2010 Santiago Ontanon and Ashwin Ram */

package d2.learn.planlearning.goalrec;

import gatech.mmpm.GameState;

import java.util.LinkedList;
import java.util.List;

import d2.plans.PetriNetPlan;


/**
 * @author kane
 * 
 * A Node in a hierarchical tree that describes how the game plan worked out.
 */
public class GameTreeNode {
	/** unique ID count */
	private static int IDcount = 0;

	/** ID for this node */
	private int id;

	/** the state of the game after the last action in the tree. */
	private GameState gs;

	/** The action that was taken at this time step */
	private PetriNetPlan plan;

	/** child nodes */
	private List<GameTreeNode> children;

	/** nodes this node depends on for preconditions */
	private List<DependencyNode> dependencies;

	/**
	 * Default constructor for leaf nodes.
	 * 
	 * @param g
	 *            the latest GameState.
	 * @param a
	 *            the last action for the GameState.
	 */
	public GameTreeNode(GameState g, PetriNetPlan p) {
		gs = g;
		plan = p;
		children = new LinkedList<GameTreeNode>();
		dependencies = new LinkedList<DependencyNode>();
		id = IDcount++;
	}

	/**
	 * Default constructor for parent nodes.
	 * @param g the final GameState.
	 * @param kids child nodes.
	 * @param dep nodes this node depends on.
	 * @param p the overall plan for this node (node plus children).
	 */
	public GameTreeNode(GameState g, List<GameTreeNode> kids, List<DependencyNode> dep,
			PetriNetPlan p) {
		gs = g;
		plan = p;
		dependencies = dep;
		children = kids;
		id = IDcount++;
	}

	/**
	 * @return the GameState
	 */
	public GameState getGs() {
		return gs;
	}

	/**
	 * @return the children
	 */
	public List<GameTreeNode> getChildren() {
		return children;
	}

	/**
	 * @return the dependencies
	 */
	public List<DependencyNode> getDependencies() {
		return dependencies;
	}

	/**
	 * @return the plan
	 */
	public PetriNetPlan getPlan() {
		return plan;
	}

	/**
	 * Shows the action and all child actions.
	 */
	public String toString() {
		String result = "(" + id + " = (";
		for (GameTreeNode child : children) {
			result += child;
		}
		result += ")";

		return result;
	}

}
