package eisbot.proxy.starmok;

import java.util.ArrayList;

import eisbot.proxy.BwapiSingleton;
import eisbot.proxy.JNIBWAPI;
import eisbot.proxy.model.Player;
import eisbot.proxy.model.Unit;
import eisbot.proxy.types.OrderType.OrderTypeTypes;
import eisbot.proxy.types.TechType.TechTypes;
import eisbot.proxy.types.UnitCommandType.UnitCommandTypes;
import eisbot.proxy.types.UnitType;
import eisbot.proxy.types.UnitType.UnitTypes;
import eisbot.proxy.types.UpgradeType.UpgradeTypes;
import gatech.mmpm.Entity;
import gatech.mmpm.Map;
import gatech.mmpm.ParseLmxTrace;
import gatech.mmpm.PhysicalEntity;
import gatech.mmpm.TwoDMap;
import eisbot.proxy.starmok.actions.*;
import eisbot.proxy.starmok.entities.Building;
import eisbot.proxy.starmok.entities.Protoss_Air;
import eisbot.proxy.starmok.entities.Resource_Mineral_Field;
import eisbot.proxy.starmok.entities.Resource_Vespene_Geyser;
import eisbot.proxy.starmok.entities.Terran_Air;
import eisbot.proxy.starmok.entities.Zerg_Air;


/**
 * Class used to convert elements between both domains: game and D2.
 * 
 * @author Michael Miranda, Zach Card
 */
public class Game2D2Converter {

	private static Map map = null;
	private static final int pixelsPerCell = 8; // should be 32 (build tiles) or 8 (walk tiles)
	
	public static void toGameAction(gatech.mmpm.Action d2Action) {
		JNIBWAPI bwapi = BwapiSingleton.getBWAPI();
		int unitID = Integer.parseInt(d2Action.getEntityID().substring(4));
		System.out.println("Action " + d2Action.getClass().getSimpleName());
		if(d2Action instanceof AttackUnit) {
			AttackUnit bwAction = (AttackUnit)d2Action;
			bwapi.attack(unitID, Integer.parseInt(bwAction.getTarget().getentityID().substring(4)));
		} else if(d2Action instanceof AttackLocation) {
			AttackLocation bwAction = (AttackLocation)d2Action;
			bwapi.attack(unitID, (int)bwAction.getCoor()[0]*pixelsPerCell, (int)bwAction.getCoor()[1]*pixelsPerCell);
		} else if(d2Action instanceof Build) {
			Build bwAction = (Build)d2Action;
			bwapi.build(unitID, (int)bwAction.getCoor()[0]*pixelsPerCell/32, (int)bwAction.getCoor()[1]*pixelsPerCell/32, UnitTypes.valueOf(bwAction.getType().getSimpleName()).ordinal());
		} else if(d2Action instanceof BuildAddon) {
			BuildAddon bwAction = (BuildAddon)d2Action;
			bwapi.buildAddon(unitID, UnitTypes.valueOf(bwAction.getType().getSimpleName()).ordinal());
		} else if(d2Action instanceof Train) {
			Train bwAction = (Train)d2Action;
			bwapi.train(unitID, UnitTypes.valueOf(bwAction.getType().getSimpleName()).ordinal());
		} else if(d2Action instanceof Research) {
			Research bwAction = (Research)d2Action;
			bwapi.research(unitID, TechTypes.valueOf(bwAction.getType()).ordinal());
		} else if(d2Action instanceof Upgrade) {
			Upgrade bwAction = (Upgrade)d2Action;
			bwapi.upgrade(unitID, UpgradeTypes.valueOf(bwAction.getType()).ordinal());
		} else if(d2Action instanceof Morph) {
			Morph bwAction = (Morph)d2Action;
			bwapi.morph(unitID, UnitTypes.valueOf(bwAction.getType().getSimpleName()).ordinal());
		} else if(d2Action instanceof Move) {
			Move bwAction = (Move)d2Action;
			bwapi.move(unitID, (int)bwAction.getCoor()[0]*pixelsPerCell, (int)bwAction.getCoor()[1]*pixelsPerCell);
		} else if(d2Action instanceof Repair) {
			Repair bwAction = (Repair)d2Action;
			bwapi.repair(unitID, Integer.parseInt(bwAction.getTarget().getentityID().substring(4)));
		} else if(d2Action instanceof Gather) {
			Gather bwAction = (Gather)d2Action;
			bwapi.gather(unitID, Integer.parseInt(bwAction.getTarget().getentityID().substring(4)));
		}
	}

	public static gatech.mmpm.Action toD2Action(Unit u) {
		JNIBWAPI bwapi = BwapiSingleton.getBWAPI();
		gatech.mmpm.Action ret = null;
		float[] Coords = {0, 0, 0};

		switch(UnitCommandTypes.values()[u.getLastCommandID()]) {
			case Attack_Unit:
				ret = new AttackUnit("unit"+u.getID(), "player"+u.getPlayerID());
				((AttackUnit)ret).setTarget(new eisbot.proxy.starmok.entities.Unit(""+u.getTargetUnitID(),"player"+bwapi.getUnit(u.getTargetUnitID()).getPlayerID()));
				break;
			case Attack_Move:
				ret = new AttackLocation("unit"+u.getID(), "player"+u.getPlayerID());
				Coords[0] = (float)(u.getTargetX()/pixelsPerCell);
				Coords[1] = (float)(u.getTargetY()/pixelsPerCell);
				((AttackLocation)ret).setCoor(Coords);
				break;
			case Build:
				ret = new Build("unit"+u.getID(), "player"+u.getPlayerID());
				((Build)ret).setType("eisbot.proxy.starmok.entities." + UnitTypes.values()[u.getBuildTypeID()].name());
				Coords[0] = (float)(u.getTargetX()/pixelsPerCell);
				Coords[1] = (float)(u.getTargetY()/pixelsPerCell);
				((Build)ret).setCoor(Coords);
				break;
			case Build_Addon:
				ret = new BuildAddon("unit"+u.getID(), "player"+u.getPlayerID());
				((BuildAddon)ret).setType("eisbot.proxy.starmok.entities." + UnitTypes.values()[u.getBuildTypeID()].name());
				Coords[0] = (float)(u.getTargetX()/pixelsPerCell);
				Coords[1] = (float)(u.getTargetY()/pixelsPerCell);
				((BuildAddon)ret).setCoor(Coords);
				break;
			case Train:
				ret = new Train("unit"+u.getID(), "player"+u.getPlayerID());
				((Train)ret).setType("eisbot.proxy.starmok.entities." + UnitTypes.values()[u.getBuildTypeID()].name());
				break;
			case Research:
				ret = new Research("unit"+u.getID(), "player"+u.getPlayerID());
				((Research)ret).setType(bwapi.getTechTypeName(u.getResearchingTechID()).replaceAll(" ", "_"));
				break;
			case Upgrade:
				ret = new Upgrade("unit"+u.getID(), "player"+u.getPlayerID());
				((Upgrade)ret).setType(bwapi.getUpgradeTypeName(u.getUpgradingUpgradeID()).replaceAll(" ", "_"));
				break;
			case Morph:
				ret = new Morph("unit"+u.getID(), "player"+u.getPlayerID());
				((Morph)ret).setType("eisbot.proxy.starmok.entities." + UnitTypes.values()[u.getBuildTypeID()].name());
				break;
			case Move:
				ret = new Move("unit"+u.getID(), "player"+u.getPlayerID());
				Coords[0] = (float)(u.getTargetX()/pixelsPerCell);
				Coords[1] = (float)(u.getTargetY()/pixelsPerCell);
				((Move)ret).setCoor(Coords);
				break;
			case Repair:
				ret = new Repair("unit"+u.getID(), "player"+u.getPlayerID());
				((Repair)ret).setTarget(new eisbot.proxy.starmok.entities.Unit("unit"+u.getTargetUnitID(),"player"+bwapi.getUnit(u.getTargetUnitID()).getPlayerID()));
				break;
			case Gather:
				ret = new Gather("unit"+u.getID(), "player"+u.getPlayerID());
				((Gather)ret).setTarget(new eisbot.proxy.starmok.entities.Unit("unit"+u.getTargetUnitID(),"player"+bwapi.getUnit(u.getTargetUnitID()).getPlayerID()));
				break;
			case Right_Click_Position:
				System.out.println("Right_Click_Position: ID = " + u.getID() + ", Order  = " + OrderTypeTypes.values()[u.getOrderID()].name());
				switch(OrderTypeTypes.values()[u.getOrderID()]) {
					case Move:
						ret = new Move("unit"+u.getID(), "player"+u.getPlayerID());
						Coords[0] = (float)(u.getTargetX()/pixelsPerCell);
						Coords[1] = (float)(u.getTargetY()/pixelsPerCell);
						((Move)ret).setCoor(Coords);
						break;
					case AttackMove:
						ret = new AttackLocation("unit"+u.getID(), "player"+u.getPlayerID());
						Coords[0] = (float)(u.getTargetX()/pixelsPerCell);
						Coords[1] = (float)(u.getTargetY()/pixelsPerCell);
						((AttackLocation)ret).setCoor(Coords);
						break;
					default:
						ret = null;
				}
				break;
			case Right_Click_Unit:
				System.out.println("Right_Click_Unit: ID = " + u.getID() + ", Order  = " + OrderTypeTypes.values()[u.getOrderID()].name());
				switch(OrderTypeTypes.values()[u.getOrderID()]) {
					case Move:
						ret = new Move("unit"+u.getID(), "player"+u.getPlayerID());
						Coords[0] = (float)(u.getTargetX()/pixelsPerCell);
						Coords[1] = (float)(u.getTargetY()/pixelsPerCell);
						((Move)ret).setCoor(Coords);
						break;
					case AttackMove:
						ret = new AttackLocation("unit"+u.getID(), "player"+u.getPlayerID());
						Coords[0] = (float)(u.getTargetX()/pixelsPerCell);
						Coords[1] = (float)(u.getTargetY()/pixelsPerCell);
						((AttackLocation)ret).setCoor(Coords);
						break;
					case AttackUnit:
						ret = new AttackUnit("unit"+u.getID(), "player"+u.getPlayerID());
						((AttackUnit)ret).setTarget(new eisbot.proxy.starmok.entities.Unit("unit"+u.getTargetUnitID(),"player"+bwapi.getUnit(u.getTargetUnitID()).getPlayerID()));
						break;
					case Harvest1:
					case Harvest2:
					case MoveToGas:
					case WaitForGas:
					case HarvestGas:
					case ReturnGas:
					case MoveToMinerals:
					case WaitForMinerals:
					case MiningMinerals:
					case ReturnMinerals:
					case Harvest3:
					case Harvest4:
					case Unload:
						ret = new Gather("unit"+u.getID(), "player"+u.getPlayerID());
						((Gather)ret).setTarget(new eisbot.proxy.starmok.entities.Unit("unit"+u.getTargetUnitID(),"player"+bwapi.getUnit(u.getTargetUnitID()).getPlayerID()));
						break;
					case Follow:
					default:
						ret = null;
				}
				break;
			default:
				//System.out.println("Unknown Action: Unit ID = " + u.getID() + ", Action ID = " + u.getLastCommandID());
				//ret = new Stop(""+u.getID(), ""+u.getPlayerID());
				break;
		}
		
		return ret;
	}
	
	
	/**
	 * Converts from the Brood War game state to the D2 game state.
	 *  
	 * @param game Current game
	 * @return GameState in
	 */
	public static gatech.mmpm.GameState toGameState(BroodWarDomain domain, int numPlayers) {
		JNIBWAPI bwapi = BwapiSingleton.getBWAPI();
		gatech.mmpm.GameState ret = new gatech.mmpm.GameState();
		ArrayList<Entity> units = new ArrayList<Entity>();
		Map currentMap = map;
		float[] coords = {0f, 0f, 0f};
		
		for(Unit unit : bwapi.getAllUnits()) {
			Entity e = unitToD2Entity(unit, bwapi.getUnitType(unit.getTypeID()), domain);
			if(e != null) {
				units.add(e);
				coords[0] = ((PhysicalEntity)e).getx();//(float)unit.getX()/pixelsPerCell;
				coords[1] = ((PhysicalEntity)e).gety();//(float)unit.getY()/pixelsPerCell;
				if(e instanceof Resource_Vespene_Geyser || 
						e instanceof Resource_Mineral_Field || 
						e instanceof Building) {
					for(float x = 0f; x < ((PhysicalEntity)e).getwidth(); x++)
					{
						for(float y = 0f; y < ((PhysicalEntity)e).getlength(); y++)
						{
							currentMap.setCellLocation('.', coords, domain);
						}
					}
				}
				else if (!(e instanceof Protoss_Air ||
						e instanceof Terran_Air ||
						e instanceof Zerg_Air))
				{
					currentMap.setCellLocation('w', coords, domain);
				}
			}
		}

		ret.addMap(currentMap);
		//ret.addMap(toD2Map(bwapi.getMap(), domain));

		/*ret.addEntity(playerToD2Entity(bwapi.getSelf()));
		/*for(Player player : bwapi.getAllies()) { // Assuming no allies at this point
			ret.addEntity(playerToD2Entity(player));
		}
		for(Player player : bwapi.getEnemies()) {
			ret.addEntity(playerToD2Entity(player));
		}
		ret.addEntity(playerToD2Entity(bwapi.getPlayer(0)));
		ret.addEntity(playerToD2Entity(bwapi.getPlayer(1)));*/
		for(int i = 0; i < numPlayers; i++)
		{
			ret.addEntity(playerToD2Entity(bwapi.getPlayer(i)));
		}
		
		for(Entity e : units) {
			ret.addEntity(e);
		}
		
		return ret;
	}
	
	private static Entity playerToD2Entity(Player player) {
		eisbot.proxy.starmok.entities.Player d2entity = new eisbot.proxy.starmok.entities.Player("player"+player.getID(), "player"+player.getID());
		d2entity.setGas(player.getGas());
		d2entity.setMinerals(player.getMinerals());
		d2entity.setSupply((player.getSupplyTotal()-player.getSupplyUsed())/2);
		d2entity.setSupply_total(player.getSupplyTotal());
		switch(player.getRaceID()) {
			case 0:
				d2entity.setRace("Zerg");
				break;
			case 1:
				d2entity.setRace("Terran");
				break;
			case 2:
				d2entity.setRace("Protoss");
				break;
			default:
				break;
		}
		String research = "";
		for(int i = 1; i <= 47; i++) {
			research = research + (player.hasResearched(i) ? 1 : 0);
		}
		d2entity.setResearch(research);
		String upgrades = "";
		for(int i = 1; i <= 63; i++) {
			upgrades = upgrades + player.upgradeLevel(i);
		}
		d2entity.setUpgrades(upgrades);
		return d2entity;
	}

	private static Entity unitToD2Entity(Unit unit, UnitType type, BroodWarDomain domain) {
		String typeName = type.getName().replaceAll(" ", "_");
		
		if(typeName.compareTo("Resource_Vespene_Geyser") == 0) {
			eisbot.proxy.starmok.entities.Resource_Vespene_Geyser geyser = new eisbot.proxy.starmok.entities.Resource_Vespene_Geyser("unit"+unit.getID(), "player"+unit.getPlayerID());
			geyser.setRemaining(unit.getResources());
			geyser.setx((int)unit.getX()/pixelsPerCell);
			geyser.sety((int)unit.getY()/pixelsPerCell);
			return geyser;
		} else if(typeName.compareTo("Resource_Mineral_Field") == 0) {
			eisbot.proxy.starmok.entities.Resource_Mineral_Field field = new eisbot.proxy.starmok.entities.Resource_Mineral_Field("unit"+unit.getID(), "player"+unit.getPlayerID());
			field.setRemaining(unit.getResources());
			field.setx((int)unit.getX()/pixelsPerCell);
			field.sety((int)unit.getY()/pixelsPerCell);
			return field;
		}
		
		//Class<eisbot.proxy.starmok.entities.Unit> typeClass;
		eisbot.proxy.starmok.entities.Unit d2entity = (eisbot.proxy.starmok.entities.Unit) domain.getEntityByName(typeName, "unit"+unit.getID(), "player"+unit.getPlayerID());
		
		/*try {
			typeClass = (Class<eisbot.proxy.starmok.entities.Unit>) Class.forName("eisbot.proxy.starmok.entities." + type.getName().replaceAll(" ", "_"));
		} catch (ClassNotFoundException e1) {
			e1.printStackTrace();
			return new eisbot.proxy.starmok.entities.Unit(""+unit.getID(), ""+unit.getPlayerID());
		}
		
		try {
			d2entity = typeClass.getConstructor(String.class, String.class).newInstance(""+unit.getID(), ""+unit.getPlayerID());
		} catch (Exception e) {
			e.printStackTrace();
			return new eisbot.proxy.starmok.entities.Unit(""+unit.getID(), ""+unit.getPlayerID());
		}*/
		
		if(d2entity == null) {
			System.out.println("Unknown unit type: " + typeName);
			//return new eisbot.proxy.starmok.entities.Unit("unit"+unit.getID(), "player"+unit.getPlayerID());
			return null;
		}
		
		d2entity.setCurrent_hitpoints(unit.getHitPoints());
		d2entity.setCurrent_energy(unit.getEnergy());
		d2entity.setx((int)unit.getX()/pixelsPerCell);
		d2entity.sety((int)unit.getY()/pixelsPerCell);
		if(d2entity instanceof eisbot.proxy.starmok.entities.Protoss_Air) {
			((eisbot.proxy.starmok.entities.Protoss_Air)d2entity).setCurrent_shields(unit.getShield());
		} else if(d2entity instanceof eisbot.proxy.starmok.entities.Protoss_Ground) {
			((eisbot.proxy.starmok.entities.Protoss_Ground)d2entity).setCurrent_shields(unit.getShield());
		}
		
		return d2entity;
	}

	private static Map toD2Map(eisbot.proxy.model.Map map, BroodWarDomain domain) {
		int coords[] = new int[2];
		int width = map.getWalkWidth();//(int)((float)map.getWalkWidth()*(float)(8/pixelsPerCell));
		int height = map.getWalkHeight();//(int)((float)map.getWalkHeight()*(float)(8/pixelsPerCell));
		TwoDMap ret = new StarmokMap(width, height, 1, 1);
		System.out.println("Width = " + width + " height = " + height);
		for (int x = 0; x < width; x++) {
			for (int y = 0; y < height; y++) {
				coords[0] = x;
				coords[1] = y;
				if(map.isWalkable(x,y)){//x*(pixelsPerCell/8), y*(pixelsPerCell/8))) {
					if(map.isBuildable(x/4,y/4)){//(int)((float)x*(float)(pixelsPerCell/32)), (int)((float)y*(float)(pixelsPerCell/32)))) {
						ret.setCellLocation('o', coords, domain);
					} else {
						ret.setCellLocation('w', coords, domain);
					}
				} else if(map.isBuildable(x/4,y/4)){//(int)((float)x*(float)(pixelsPerCell/32)), (int)((float)y*(float)(pixelsPerCell/32)))) {
					ret.setCellLocation('b', coords, domain);
				} else {
					ret.setCellLocation('.', coords, domain);
				}
			}
		}
		
		return ret;
	}
	
	public static int CanUseTech(int unitID, String tech) {
		JNIBWAPI bwapi = BwapiSingleton.getBWAPI();
		int techID = eisbot.proxy.types.TechType.TechTypes.valueOf(tech).ordinal();
		if(!bwapi.getSelf().hasResearched(techID)) return 0;
		if(bwapi.getTechType(techID).getEnergyUsed() > bwapi.getUnit(unitID).getEnergy()) return 0;
		return 1;
	}
	
	public static void setMap(JNIBWAPI bwapi, BroodWarDomain domain) {
		map = toD2Map(bwapi.getMap(), domain);
	}

	static ParseLmxTrace parser = new ParseLmxTrace(BroodWarDomain.getDomainName());
}
