
package eisbot.proxy.starmok;

import org.jdom.Element;

import gatech.mmpm.Action;
import gatech.mmpm.ActionParameter;
import gatech.mmpm.Entity;
import gatech.mmpm.GameState;
import gatech.mmpm.TwoDMap;
import d2.execution.adaptation.PlanAdaptation;
import d2.execution.adaptation.parameters.EagerGameStatePotentialField;
import d2.execution.adaptation.parameters.GameStatePotentialField;
import d2.execution.adaptation.parameters.ParameterAdaptation;

public class StarmokParameterAdaptation extends ParameterAdaptation {
	
	public StarmokParameterAdaptation(String player) {
        super(player);
    }
	
	protected float[] adaptCoordinates(float original_coords[], String originalUnitID, GameState original_gs, String currentUnitID, int current_cycle, GameState current_gs, Action a, ActionParameter actionParameter) {
    	if (DEBUG >= 1) System.out.println("About to adapt coordinates.");
    	Entity originalUnit = original_gs.getEntity(originalUnitID);
        Entity currentUnit = current_gs.getEntity(currentUnitID);
        String originalPlayer = originalUnit.getowner();
        String currentPlayer = currentUnit.getowner();
        GameStatePotentialField original_pf = (GameStatePotentialField) original_gs.getMetaData("GameStatePotentialField-" + originalPlayer);
        GameStatePotentialField current_pf = (GameStatePotentialField) current_gs.getMetaData("GameStatePotentialField-" + currentPlayer);
        int mapSize = current_gs.getMap().size();
        int bestPos = -1;
        double bestDistance = 0, distance;
        int pos1 = original_gs.getMap().toCell(original_coords);

        if (original_pf == null) {
            original_pf = new EagerGameStatePotentialField(original_gs, originalPlayer);
            original_gs.addMetaData("GameStatePotentialField-" + originalPlayer, original_pf);
        }
        if (current_pf == null) {
            current_pf = new EagerGameStatePotentialField(current_gs, currentPlayer);
            current_gs.addMetaData("GameStatePotentialField-" + currentPlayer, current_pf);
        }

        if (DEBUG_COORDINATES >= 1) {
            System.out.println("\n--------------------------------------------------------------------");
        }

        float minx = original_coords[0] - 10;
        if (minx < 0) minx = 0f;
        float miny = original_coords[1] - 10;
        if (miny < 0) miny = 0f;
        float maxx = original_coords[0] + 10;
        if (maxx >= ((TwoDMap)current_gs.getMap()).getSizeInDimension(0)) maxx = ((TwoDMap)current_gs.getMap()).getSizeInDimension(0)-1;
        float maxy = original_coords[1] + 10;
        if (maxy >= ((TwoDMap)current_gs.getMap()).getSizeInDimension(0)) maxy = ((TwoDMap)current_gs.getMap()).getSizeInDimension(1)-1;
        
        /*for (float x = minx; x < maxx; x++) {
        	for (float y = miny; y < maxy; y++) {
        		float new_coords[] = {x,y};
        		int pos2 = current_gs.getMap().toCell(new_coords);*/
        for (int pos2 = 0; pos2 < mapSize; pos2++) {
        	float new_coords[] = current_gs.getMap().toCoords(pos2);
        	if (new_coords[0] < minx || new_coords[0] > maxx || new_coords[1] < miny || new_coords[1] > maxy) continue;
        		distance = original_pf.distance(pos1, current_pf, pos2, null, m_d2.getWorldModel().getEntityWeights());
	            if (DEBUG_COORDINATES >= 2) {
	                for (int i = 0; i < new_coords.length; i++) {
	                    System.out.print(new_coords[i] + ",");
	                }
	                System.out.println(": " + distance);
	            }
	
	            if (bestPos == -1 || distance < bestDistance) {
	
	                    Action ca = (Action) a.clone();
	                    ca.setParameterValue(actionParameter.m_name, actionParameter.m_type.toString(new_coords));
	                    if (ca.checkValidCondition(current_cycle, current_gs, ca.getPlayerID())) {
	                        bestPos = pos2;
	                        bestDistance = distance;
	                    }
	            }
	        /*}*/
        }
        if (DEBUG >= 1) System.out.println("Adapted coordinates.");

        if (bestPos == -1) {
            return super.adaptCoordinates(original_coords, originalUnitID, original_gs, currentUnitID, current_cycle, current_gs, a, actionParameter);
        }
        return current_gs.getMap().toCoords(bestPos);
    }
	
	public static PlanAdaptation loadFromXMLInternal(Element xml, String a_player) {
        return new StarmokParameterAdaptation(a_player);
    }
}