
package eisbot.proxy.starmok;

import gatech.mmpm.GameState;
import gatech.mmpm.sensor.Sensor;
import gatech.mmpm.util.Pair;

import java.util.List;

import d2.core.D2;
import d2.execution.adaptation.PlanAdaptation;
import d2.execution.planexecution.RealTimePlanExecution;
import d2.execution.planner.Planner;
import d2.plans.ActionPlan;
import d2.plans.FailureTransition;
import d2.plans.PetriNetPlan;
import d2.plans.Plan;
import d2.plans.PlanPreconditionFailedState;
import d2.plans.PreFailureTransition;
import d2.plans.PreTransition;
import d2.plans.State;
import d2.plans.SuccessTransition;
import d2.plans.Transition;

public class StarmokPlanExecution extends RealTimePlanExecution {
    
    private int lastAdapted = -1;
    private Plan lastAdaptation = null;
    
	public StarmokPlanExecution(String player, PlanAdaptation parameterAdapter, PlanAdaptation planAdapter, D2 d2) {
        super(player, parameterAdapter, planAdapter, d2);
    }

    public StarmokPlanExecution() {
        super(null, null, null, null);
    }
    
    private boolean executeStateChainTillAction(Plan topLevelPlan, PetriNetPlan a_plan, State s, List<Plan> a_plansToExecute, int cycle, GameState a_gs, String player) {
        boolean tokensMoving = false;
        Plan adaptedPlan = null, originalPlan = null;
        boolean previouslyAdapted = true;

        if (EXECUTION_DEBUG >= 2) {
            System.out.println("\texecutePetriNetFAST: executeStateChainTillAction: in state " + s.getElementID() + " tokens: " + s.getCurrentNumberOfTokens());
        }

        for (Pair<Integer, Transition> t_p : s.getNextTransitions()) {
            if (s.getCurrentNumberOfTokens() >= t_p._a) {

//                System.out.println("Transition type: " + t_p._b.getClass().getSimpleName());

                originalPlan = t_p._b.getPlan();
                adaptedPlan = null;
                if (t_p._b instanceof PreTransition) {
                    // If we are about to start a subplan, adapt it!
                    if (EXECUTION_DEBUG >= 1) {
                        System.out.println("About to Adapt: " + originalPlan);
                        System.out.println("Units in use are: " + unitsInUse);
                    }
                    /**/if (originalPlan instanceof ActionPlan)
                    {
                    	if (originalPlan.hashCode() != lastAdapted)
                    	{
                    		adaptedPlan = m_parameterAdapter.adapt(originalPlan, originalPlan.getOriginalGameState(), originalPlan.getOriginalPlayer(), null, cycle, a_gs, player, null,unitsInUse);
                    		lastAdaptation = adaptedPlan;
                    		lastAdapted = originalPlan.hashCode();
                    		previouslyAdapted = false;
                    	}
                    	else
                    	{
                    		if (EXECUTION_DEBUG >= 1) System.out.println("Action previously adapted");
                    		adaptedPlan = lastAdaptation;
                    	}
                    }
                    else/**/
                    	adaptedPlan = m_parameterAdapter.adapt(originalPlan, originalPlan.getOriginalGameState(), originalPlan.getOriginalPlayer(), null, cycle, a_gs, player, null,unitsInUse);
                    if (adaptedPlan == null) {
//						t_p.m_b.setPlan(adapted);
                        if (EXECUTION_DEBUG >= 1) {
                            System.out.println("executeStateChainTillAction: stopped, since the plan cannot be adapted.");
                        }
//                        return tokensMoving;
                    } else {
                        if (EXECUTION_DEBUG >= 1) System.out.println("Action successfully adapted");
                    }
                }

                // If adaptedPlan != null, that means that the transition is a PRETRANSITION, and the Sensor is thus
                // the precondition of the plan
                if ((adaptedPlan != null && adaptedPlan.checkPreCondition(cycle, a_gs, player))
                    || (adaptedPlan == null && t_p._b instanceof SuccessTransition && t_p._b.getPlan().checkSuccessCondition(cycle, a_gs, player))
                    || (adaptedPlan == null && t_p._b instanceof FailureTransition && t_p._b.getPlan().checkFailureCondition(cycle, a_gs, player))
                    || (adaptedPlan == null && t_p._b instanceof PreFailureTransition && t_p._b.getPlan().checkPreFailureCondition(cycle, a_gs, player))
                    || (adaptedPlan == null && t_p._b.getPlan() == null && (Float) t_p._b.getCondition().evaluate(cycle, a_gs, player, a_plan.getContext(cycle, a_gs, player)) >= Sensor.BOOLEAN_TRUE_THRESHOLD)) {

                    if (adaptedPlan != null) {
                        if (adaptedPlan instanceof ActionPlan) {
                        	/**/if (previouslyAdapted) adaptedPlan = m_parameterAdapter.adapt(originalPlan, originalPlan.getOriginalGameState(), originalPlan.getOriginalPlayer(), null, cycle, a_gs, player, null,unitsInUse);
                        	lastAdapted = -1;/**/
                            unitsInUse.add(((ActionPlan)adaptedPlan).getAction().getEntityID());
                        }
                        // Replace this plan in the planstate, pretransition, successtransition, and failuretransition
                        a_plan.replacePlan(t_p._b.getPlan(), adaptedPlan);
                        if (EXECUTION_DEBUG >= 1) {
                            System.out.println("originalPlan: " + originalPlan);
                        }
                        if (EXECUTION_DEBUG >= 1) {
                            System.out.println("adaptedPlan: " + adaptedPlan);
                        }
                    }

                    if (t_p._b instanceof SuccessTransition ||
                        t_p._b instanceof FailureTransition) {
                        Plan p = t_p._b.getPlan();
                        if (p!=null && p instanceof ActionPlan) {
                            unitsInUse.remove(((ActionPlan)p).getAction().getEntityID());
                        }
                    }

//				 	System.out.println("Condition " + t_p.m_b.getCondition() + " satisfied of a " + t_p.m_b.getClass().getSimpleName() + " [cycle: " + cycle + "]");

//					If it's a precondition transition, execute the plan:
//					and its an action (or subgoalplan, or petrinetplan...whatever), so well...here goes the action...

                    if (EXECUTION_DEBUG >= 1) {
                        if (t_p._b instanceof PreFailureTransition
                            || t_p._b instanceof PreTransition
                            || t_p._b instanceof SuccessTransition
                            || t_p._b instanceof FailureTransition) {
                            System.out.println(t_p._b.getClass().getSimpleName() + " fired for plan " + t_p._b.getPlan());
                            System.out.println("With context: " + t_p._b.getPlan().getContext(cycle, a_gs, player));
                            if (t_p._b.getPlan() instanceof ActionPlan) {
                                System.out.println("Unit: " + a_gs.getEntity(((ActionPlan)t_p._b.getPlan()).getAction().getEntityID()));
                            }
                            System.out.println(t_p._b.getCondition());

                            // clear the potential fields for the action just executed:
                            if (t_p._b instanceof SuccessTransition
                                    || t_p._b instanceof FailureTransition) {
                                Plan p = t_p._b.getPlan();
                                GameState gs = p.getOriginalGameState();
                                gs.clearMetadata();
                            }

                            if (EXECUTION_DEBUG >= 2) {
                                System.out.println(" /- cycle " + cycle + " --------------------\\ ");
                                Planner.printPlanNice(topLevelPlan);
                                System.out.println(" \\-------------------------/ ");
                            }

                        }
                    }

                    if (t_p._b instanceof PreTransition) {
                        if (EXECUTION_DEBUG >= 2) {
                            System.out.println("\texecutePetriNetFAST:Fired Transition" + t_p._b.getElementID() + " removed tokens from state " + s.getElementID() + " -> " + s.getCurrentNumberOfTokens() + " (condition: " + t_p._b.getCondition() + ", plan: " + t_p._b.getPlan() + ")");
                        }

                        if (t_p._b.getPlan() != null) {
                            a_plansToExecute.add(t_p._b.getPlan());
                        } else {
                            System.err.println("Plan null in petri net element! -> " + t_p._b.getElementID());
                        }
                        tokensMoving = true;
                        s.setCurrentNumberOfTokens(s.getCurrentNumberOfTokens() - t_p._a);
//                        System.out.println("Tokens in state " + s.getElementID() + " now " + s.getCurrentNumberOfTokens() + " (down " + t_p._a + ")");
                        for (Pair<Integer, State> s_p : t_p._b.getNextStates()) {
                            s_p._b.setCurrentNumberOfTokens(s_p._b.getCurrentNumberOfTokens() + s_p._a);
//                            System.out.println("Tokens in state " + s_p._b.getElementID() + " now " + s_p._b.getCurrentNumberOfTokens() + " (up " + s_p._a + ")");
                            if (EXECUTION_DEBUG >= 2) {
                                System.out.println("\texecutePetriNetFAST:Added " + t_p._a + " token to state " + s_p._b.getElementID());
                            }
                        }
                        return tokensMoving;
                    } else {
                        if (EXECUTION_DEBUG >= 2) {
                            System.out.println("\texecutePetriNetFAST:Fired Transition" + t_p._b.getElementID() + " removed tokens from state " + s.getElementID() + " -> " + s.getCurrentNumberOfTokens() + " (condition: " + t_p._b.getCondition() + ", plan: " + t_p._b.getPlan() + ")");
                        }

                        // Remove the tokens from the preceding state:
                        tokensMoving = true;
                        s.setCurrentNumberOfTokens(s.getCurrentNumberOfTokens() - t_p._a);
//                        System.out.println("Tokens in state " + s.getElementID() + " now " + s.getCurrentNumberOfTokens() + " (down " + t_p._a + ")");
                        for (Pair<Integer, State> s_p : t_p._b.getNextStates()) {
                            s_p._b.setCurrentNumberOfTokens(s_p._b.getCurrentNumberOfTokens() + s_p._a);
//                            System.out.println("Tokens in state " + s_p._b.getElementID() + " now " + s_p._b.getCurrentNumberOfTokens() + " (up " + s_p._a + ")");
                            executeStateChainTillAction(topLevelPlan, a_plan, s_p._b, a_plansToExecute, cycle, a_gs, player);
                            if (s_p._b instanceof PlanPreconditionFailedState
                                    && s_p._b.getNextTransitions().size() == 0) {
                                // The preconditions are not satisfied, and the action has not been adapted before:
                                plansThatCouldntStart.add(new Pair<Plan, Plan>(s_p._b.getPlan(), a_plan));
                            }
                        }
                    }
                } else {
                    if (EXECUTION_DEBUG >= 2) {
                        if (adaptedPlan == null) {
                            System.out.println("AdaptedPlan = null");
                            System.out.println("OriginalPlan = " + originalPlan);
                            if (t_p._b instanceof SuccessTransition) {
                                Sensor sensor = t_p._b.getCondition();
                                System.out.println("executePetriNetFAST: waiting for SuccessCondition " + sensor + " [cycle: " + cycle + "]");
                                System.out.println("Which evaluates to: " + t_p._b.getPlan().checkSuccessCondition(cycle, a_gs, player));
                            } else if (t_p._b instanceof FailureTransition) {
                                Sensor sensor = t_p._b.getCondition();
                                System.out.println("executePetriNetFAST: waiting for FailureCondition " + sensor + " [cycle: " + cycle + "]");
                                System.out.println("Which evaluates to: " + t_p._b.getPlan().checkFailureCondition(cycle, a_gs, player));
                            } else {
                                Sensor sensor = t_p._b.getCondition();
                                System.out.println("executePetriNetFAST: waiting for Sensor " + sensor + " [cycle: " + cycle + "]");
                                System.out.println("Which evaluates to: " + sensor.evaluate(cycle, a_gs, player));
                            }
                        } else {
                            Sensor sensor = adaptedPlan.getPreCondition();
                            System.out.println("executePetriNetFAST: waiting for PreCondition " + sensor + ", plan: " + adaptedPlan + " [cycle: " + cycle + "]");
                            System.out.println("Which evaluates to: " + adaptedPlan.checkPreCondition(cycle, a_gs, player));
                        }
                    }
                }
            }
        }
        return tokensMoving;
    }
}