package eisbot.proxy.starmok;
/**
 * Example of a Java AI Client that does nothing.
 */
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import eisbot.proxy.BwapiSingleton;
import eisbot.proxy.JNIBWAPI;
import eisbot.proxy.model.Player;
import eisbot.proxy.model.Unit;
import eisbot.proxy.starmok.BroodWarDomain;
import eisbot.proxy.starmok.Game2D2Converter;
import gatech.mmpm.Action;
import gatech.mmpm.ConfigurationException;
import gatech.mmpm.GameState;
public class StarmokTracer implements gatech.mmpm.learningengine.IMEExecutor.IGameStateProvider {

	private BroodWarDomain domain;
	private gatech.mmpm.tracer.ITracer tracer;
	private HashMap<Unit, Integer> last_commands;
	private List<Action> actions;
	private GameState state;
	private int trace_interval = 25;
	private int normal_speed = 0; // if speed > 0, lower is faster (any negative value resets to normal speed, 0 goes as fast as possible)
	private int cycle;
	private int numPlayers = 0;
	
	public StarmokTracer() {}
	
	public StarmokTracer(int interval, int speed) {
		trace_interval = interval;
		normal_speed = speed;
	}
	
	public void startTrace() {
		domain = new BroodWarDomain();
		last_commands = new HashMap<Unit, Integer>();
		actions = new LinkedList<Action>();
		JNIBWAPI bwapi = BwapiSingleton.getBWAPI();
		BwapiSingleton.setWinner(-1);
		Game2D2Converter.setMap(bwapi, domain);
		numPlayers = getNumPlayers();
		if(d2.core.Config.getDomainName() == null) {
			try {
				d2.core.Config.setDomain(domain);
			} catch (ConfigurationException e1) {
				e1.printStackTrace();
			}
		}
		try {
			tracer = gatech.mmpm.tracer.TracerFactory.BuildTracer("file");
		} catch (ConfigurationException e) {
			e.printStackTrace();
		}
		if(tracer != null) {
			System.out.println("Starting trace.");
			tracer.beginTrace();
			java.util.Properties prop = new java.util.Properties();
			prop.setProperty("domain", d2.core.Config.getDomainName());
			prop.setProperty("map", bwapi.getMap().getName());
			{
				String players_string = "";/*<player>player0</player><player>player1</player>";
				players_string += "<player>player" + bwapi.getSelf().getID() + "</player>";
				for (Player p : bwapi.getEnemies()) {
					players_string += "<player>player" + p.getID() + "</player>";
				}
				for (Player p : bwapi.getAllies()) {
					players_string += "<player>player" + p.getID() + "</player>";
				}*/
				for(int i = 0; i < numPlayers; i++)
				{
					players_string += "<player>player" + i + "</player>";
				}
				prop.setProperty("players", players_string);
			}
			tracer.putMetadata(prop);
		} else {
			System.out.println("Could not create tracer! :(");
		}
		//System.out.println("Map height = " + bwapi.getMap().getHeight() + ", walk height = "  + bwapi.getMap().getWalkHeight());
		//System.out.println("Map width = " + bwapi.getMap().getWidth() + ", walk width = "  + bwapi.getMap().getWalkWidth());
	}
	
	public void doTrace()
	{
		JNIBWAPI bwapi = BwapiSingleton.getBWAPI();
		actions.clear();
		for(Unit u : bwapi.getAllUnits())
		{
			if(last_commands.containsKey(u)) {
				if(last_commands.get(u) != u.getLastCommandFrame()) {
					actions.add(Game2D2Converter.toD2Action(u));
					last_commands.put(u, new Integer(u.getLastCommandFrame()));
				}
			} else {
				actions.add(Game2D2Converter.toD2Action(u));
				last_commands.put(u, new Integer(u.getLastCommandFrame()));
			}
		}
		cycle = bwapi.getFrameCount();
		if(tracer != null && (cycle % trace_interval == 1 || (!actions.isEmpty() && cycle > 1))) {
			//System.out.println("Starting trace for cycle " + cycle);
			bwapi.setGameSpeed(10000);
			tracer.beginGameCycle(cycle);
			state = Game2D2Converter.toGameState(domain, numPlayers);
			tracer.putGameState(state);
			for(Action a : actions) {
				if(a != null) tracer.putAction(a);
			}
			tracer.endGameCycle();
			//System.out.println("Finished trace for cycle " + cycle);
			bwapi.setGameSpeed(normal_speed);
		}
	}
	
	public void endTrace(boolean winner) {
		String winnerName = "Opponent";
		if(winner) {
			int myID = BwapiSingleton.getBWAPI().getSelf().getID();
			winnerName = "player"+myID;
			eisbot.proxy.starmok.entities.Player self = (eisbot.proxy.starmok.entities.Player) state.getEntity(winnerName);
			self.setHasWon(1);
			state.deleteEntity(winnerName);
			state.addEntity(self);
		}
		if(tracer != null) {
			tracer.beginGameCycle(cycle+1);
			tracer.putGameState(state);
			tracer.endGameCycle();
			tracer.endTrace(domain, winnerName);
			System.out.println("Ended trace.");
		}
	}
	
	public List<Action> getActions() {
		return actions;
	}
	
	public GameState getState() {
		return state;
	}

	@Override
	public GameState getGameState() {
		return state;
	}

	@Override
	public int getGameCycle() {
		return cycle;
	}
	
	public BroodWarDomain getDomain() {
		return domain;
	}
	
	private int getNumPlayers()
	{
		int i = 0;
		JNIBWAPI bwapi = BwapiSingleton.getBWAPI();
		while (bwapi.getPlayer(i) != null) i++;
		return i;
	}
}
